<?php
/**
 * @file
 * This file contains the admnistrative UI.
 */

/**
 * Page callback: Handles multistep precessing.
 *
 * @return string
 *   Multistep wizard output.
 */
function odata_endpoints_form($step = NULL) {

  // Define array for ctools multistep wizard.
  $form_info = array(
    'id' => 'odata_endpoints_form',
    'path' => "admin/structure/odata/add/%step",
    'show trail' => FALSE,
    'show back' => TRUE,
    'show return' => FALSE,
    'next callback' => 'odata_endpoints_form_wizard_next',
    'finish callback' => 'odata_endpoints_form_wizard_finish',
    'cancel callback' => 'odata_endpoints_form_wizard_cancel',
    // Define forms order.
    'order' => array(
      'first' => t("Enter endpoint's info"),
      'second' => t("Select collections"),
    ),
    // Define forms.
    'forms' => array(
      'first' => array(
        'form id' => 'odata_create_endpoints_form_first_step',
      ),
      'second' => array(
        'form id' => 'odata_create_endpoints_form_second_step',
      ),
    ),
  );

  // We skip all this here by just using an id of 1.
  $object_id = 'odata_endpoints_form';

  if (empty($step)) {

    // We reset the form when $step is NULL because that means they have
    // for whatever reason started over.
    odata_endpoints_form_cache_clear($object_id);
    $step = 'first';
  }

  // This automatically gets defaults if there wasn't anything saved.
  $object = odata_endpoints_form_cache_get($object_id);

  // Live $form_state changes.
  $form_state = array(
    'ajax' => FALSE,
    // Put our object and ID into the form state cache so we can easily find it.
    'object_id' => $object_id,
    'object' => &$object,
  );

  // Send this all off to our form. This is like drupal_get_form only wizardy.
  ctools_include('wizard');
  $form = ctools_wizard_multistep_form($form_info, $step, $form_state);
  return drupal_render($form);
}

/**
 * Handles the 'next' click on the add/edit pane form wizard.
 *
 * All we need to do is store the updated pane in the cache.
 */
function odata_endpoints_form_wizard_next(&$form_state) {
  odata_endpoints_form_cache_set($form_state['object_id'], $form_state['object']);
}

/**
 * Handles the 'finish' click on teh add/edit pane form wizard.
 *
 * All we need to do is set a flag so the return can handle adding
 * the pane.
 */
function odata_endpoints_form_wizard_finish(&$form_state) {

  $form_state['complete'] = TRUE;
  $form_state['redirect'] = url('admin/structure/odata', array('absolute' => TRUE));

  $options = array(
    'collection' => $form_state['object']->odata_collection,
    'collections_schema' => $form_state['object']->odata_collections_schema,
    'accept_request_header' => $form_state['object']->odata_accept_request_header,
  );

  odata_merges_endpoint(
    array(
      'endpoint_name' => $form_state['object']->odata_endpoint_title_machine_name,
    ),
    array(
      'title' => $form_state['object']->odata_endpoint_title,
      'url' => $form_state['object']->odata_endpoint_uri,
      'options' => serialize($options),
    ),
    array(
      'endpoint_name' => $form_state['object']->odata_endpoint_title_machine_name,
      'title' => $form_state['object']->odata_endpoint_title,
      'url' => $form_state['object']->odata_endpoint_uri,
      'options' => serialize($options),
    )
  );

  // Manually clear views cache to get latest changes.
  views_invalidate_cache();
  drupal_set_message(t('Your settings have been saved.'));
}
/**
 * Clears the wizard cache.
 *
 * @param int $id
 *   cache id.
 */
function odata_endpoints_form_cache_clear($id) {
  ctools_include('object-cache');
  ctools_object_cache_clear('odata_endpoints_form', $id);
}

/**
 * Stores our little cache so that we can retain data from form to form.
 *
 * @param int $id
 *   cache id.
 * @param object $object
 *   object with form values.
 */
function odata_endpoints_form_cache_set($id, $object) {
  ctools_include('object-cache');
  ctools_object_cache_set('odata_endpoints_form', $id, $object);
}

/**
 * Gets the current object from the cache, or default.
 *
 * @param int $id
 *   cache id.
 *
 * @return object
 *   cache with stored stuff.
 */
function odata_endpoints_form_cache_get($id) {
  ctools_include('object-cache');
  $object = ctools_object_cache_get('odata_endpoints_form', $id);
  if (!$object) {
    // Create a default object.
    $object = new stdClass();
  }

  return $object;
}

/**
 * First step of creating a new endpoint wizard.
 *
 * Here we add a new title & machine name and also the
 * URI to use to.
 */
function odata_create_endpoints_form_first_step($form, &$form_state) {

  $form['odata_endpoint_title'] = array(
    '#type' => 'textfield',
    '#title' => t('Title'),
    '#description' => t('A unique name to construct the primary name in the Views UI. <br/> It must only contain lowercase letters, numbers and hyphens.'),
    '#required' => TRUE,
    '#default_value' => isset($form_state['object']->odata_endpoint_title) ? $form_state['object']->odata_endpoint_title : NULL,
  );

  $form['odata_endpoint_title_machine_name'] = array(
    '#type' => 'machine_name',
    '#title' => t("Machine Name"),
    '#required' => TRUE,
    '#description' => t("machine-friendly name."),
    '#size' => 15,
    '#maxlength' => 15,
    '#default_value' => isset($form_state['object']->odata_endpoint_title_machine_name) ? $form_state['object']->odata_endpoint_title_machine_name : NULL,
    '#machine_name' => array(
      'exists' => 'odata_check_machine_name_if_exist',
      'source' => array('odata_endpoint_title'),
    ),
  );

  $form['odata_endpoint_uri'] = array(
    '#type' => 'textfield',
    '#title' => t('Endpoint URI'),
    '#description' => t('The URL of the endpoint, e.g., http://example.com/odataservice.svc'),
    '#default_value' => isset($form_state['object']->odata_endpoint_uri) ? $form_state['object']->odata_endpoint_uri : NULL,
    '#required' => TRUE,
  );

  $form['odata_accept_request_header'] = array(
    '#type' => 'radios',
    '#title' => t('Accept Request Header'),
    '#options' => array(
      'application/json' => 'JSON',
    ),
    '#required' => TRUE,
    '#default_value' => isset($form_state['object']->odata_accept_request_header) ? $form_state['object']->odata_accept_request_header : 'application/json',
    '#description' => t('Indicates that the request is specifically limited to the desired type.'),
  );

  return $form;
}

/**
 * Validates odata_create_endpoints_form_first_step.
 */
function odata_create_endpoints_form_first_step_validate($form, &$form_state) {

  if (!valid_url($form_state['values']['odata_endpoint_uri'], TRUE)) {
    form_set_error('odata_endpoint_uri', t('Enter a valid uri. e.g., http://example.com/odataservice.svc'));
  }

  // Find the available collections, if any.
  $reply = odata_request($form_state['values']['odata_endpoint_uri'] . '/$metadata');

  // If there is an error, say so.
  if (!isset($reply)) {
    form_set_error('odata_endpoint_uri', t("Please ensure that your checkpoint is up and running."));
  }
  else {
    $form_state['object']->odata_schema = $reply;
  }
}

/**
 * Handles submit of odata_create_endpoints_form_first_step.
 */
function odata_create_endpoints_form_first_step_submit($form, &$form_state) {

  $uri = $form_state['values']['odata_endpoint_uri'];

  // Remove / from the end of the URI.
  if (drupal_substr($uri, -1) == '/') {
    $uri = rtrim($uri, '/');
  }

  $form_state['object']->odata_endpoint_uri = $uri;
  $form_state['object']->odata_endpoint_title = $form_state['values']['odata_endpoint_title'];
  $form_state['object']->odata_endpoint_title_machine_name = $form_state['values']['odata_endpoint_title_machine_name'];
  $form_state['object']->odata_accept_request_header = $form_state['values']['odata_accept_request_header'];
}

/**
 * Second step of selecting which collection to use.
 *
 * Here we fetch the available collections().
 */
function odata_create_endpoints_form_second_step($form, &$form_state) {

  $collections = new OdataParser($form_state['object']->odata_schema);
  $entity_names = $collections->GetEntityTypes();

  if (!empty($entity_names)) {
    $form['collection'] = array(
      '#type' => 'radios',
      '#options' => $entity_names,
      '#required' => TRUE,
      '#title' => t('Available collection sets'),
      '#description' => t('Select the collection you want to add.'),
    );

    $form['collections_schema'] = array(
      '#type' => 'hidden',
      '#value' => serialize($collections->GetPropertiesPerEntity()),
    );
  }
  else {
    drupal_set_message(t("We weren't able to find any available collection sets"), 'error');
  }

  return $form;
}

/**
 * Submit handler for odata_create_endpoints_form_second_step().
 */
function odata_create_endpoints_form_second_step_submit($form, &$form_state) {
  $form_state['object']->odata_collection = $form_state['values']['collection'];
  $form_state['object']->odata_collections_schema = $form_state['values']['collections_schema'];
}

/**
 * Returns a list of all oData Endpoints.
 */
function odata_get_endpoints_form() {

  $result = db_query('SELECT endpoint_name, url, title FROM {odata_endpoints}');

  if ($result->rowCount() <= 0) {
    return t('You have not create any oData endpoints yet.');
  }

  $header = array(
    array('data' => 'Title'),
    array('data' => 'URL'),
    array('data' => 'Operations', 'colspan' => '2'),
  );

  $rows = array();
  foreach ($result as $endpoint) {
    $row = array(
      array('data' => $endpoint->title . " <small>(Machine name: $endpoint->endpoint_name)</small>"),
      array('data' => $endpoint->url),
    );

    $row[] = array('data' => l(t('Edit'), 'admin/structure/odata/endpoint/' . $endpoint->endpoint_name . '/edit'));
    $row[] = array('data' => l(t('Delete'), 'admin/structure/odata/endpoint/' . $endpoint->endpoint_name . '/delete'));
    $rows[] = $row;
  }

  return theme('table', array('header' => $header, 'rows' => $rows));
}

/**
 * OData Delete Endpoint callback.
 *
 * Check access and get a confirm form for deletion of an endpoint menu.
 */
function odata_delete_endpoints_delete($endpoint) {
  return drupal_get_form('odata_delete_endpoints_confirm', $endpoint['endpoint_name']);
}

/**
 * Build a confirm form for deletion of an odata endpoint menu.
 */
function odata_delete_endpoints_confirm($form, &$form_state, $machine_name) {

  $caption = '<p>' . t('This action cannot be undone.') . '</p>';

  return confirm_form($form, t('Are you sure you want to delete the oData endpoint @title?', array('@title' => $machine_name)), 'admin/structure/odata/list', $caption, t('Delete'));
}

/**
 * Deletes an oData endpoint.
 */
function odata_delete_endpoints_confirm_submit($form, &$form_state) {

  $form_state['redirect'] = 'admin/structure/odata/list';
  $title = $form_state['build_info']['args'][0];

  // Delete the endpoint.
  db_delete('odata_endpoints')->condition('endpoint_name', $title)->execute();

  drupal_set_message(t('Deleted oData endpoint @title.', array('@title' => $title)));
  watchdog('odata', 'Deleted oData endpoint @title.', array('@title' => $title));
}

/**
 * First step of creating a new endpoint wizard.
 *
 * Here we add a new title & machine name and also the
 * URI to use to.
 */
function odata_endpoints_edit_form($form, &$form_state, $endpoint) {

  $endpoint_options = unserialize($endpoint['options']);

  $form['odata_endpoint_title'] = array(
    '#type' => 'textfield',
    '#title' => t('Title'),
    '#description' => t('A unique name to construct the primary name in the Views UI. <br/> It must only contain lowercase letters, numbers and hyphens.'),
    '#required' => TRUE,
    '#default_value' => $endpoint['title'],
  );

  $form['odata_endpoint_uri'] = array(
    '#type' => 'textfield',
    '#title' => t('Endpoint URI'),
    '#description' => t('The URL of the endpoint, e.g., http://example.com/odataservice/'),
    '#default_value' => $endpoint['url'],
    '#required' => TRUE,
    '#disabled' => TRUE,
  );

  // Find the available collections, if any.
  $reply = odata_request($endpoint['url'] . '/$metadata', 'application/xml');

  // If there is an error, please abort.
  if (is_null($reply)) {
    return array();
  }

  $collections = new OdataParser($reply);
  $entity_types = $collections->GetEntityTypes();

  if (!empty($entity_types)) {

    $form['odata_accept_request_header'] = array(
      '#type' => 'radios',
      '#title' => t('Accept Request Header'),
      '#options' => array(
        'application/json' => 'JSON',
      ),
      '#required' => TRUE,
      '#default_value' => $endpoint_options['accept_request_header'],
      '#description' => t('Indicates that the request is specifically limited to the desired type.'),
    );

    $form['odata_collection'] = array(
      '#type' => 'radios',
      '#default_value' => $endpoint_options['collection'],
      '#options' => $entity_types,
      '#required' => TRUE,
      '#title' => t('Available collection sets'),
      '#description' => t('Select the collection you want to add.'),
    );

    $form['odata_collections_schema'] = array(
      '#type' => 'hidden',
      '#default_value' => $endpoint_options['collections_schema'],
      '#value' => serialize($collections->GetPropertiesPerEntity()),
    );
  }
  else {
    drupal_set_message(t("We weren't able to find any available collection sets"), 'error');
  }

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save endpoint'),
  );

  $form['cancel'] = array(
    '#markup' => l(t('Cancel'), url('admin/structure/odata/endpoint', array('absolute' => TRUE))),
    '#weight' => 20,
  );

  return $form;
}

/**
 * Validates the edit form for an oData.
 */
function odata_endpoints_edit_form_validate($form, &$form_state) {
  if (!valid_url($form_state['values']['odata_endpoint_uri'], TRUE)) {
    form_set_error('odata_endpoint_uri', t('Enter a valid uri. e.g., http://example.com/odataservice/'));
  }
}

/**
 * Updates an oData endpoint.
 */
function odata_endpoints_edit_form_submit($form, &$form_state) {

  $options = array(
    'collection' => $form_state['values']['odata_collection'],
    'collections_schema' => $form_state['values']['odata_collections_schema'],
    'accept_request_header' => $form_state['values']['odata_accept_request_header'],
  );

  odata_merges_endpoint(
    array(
      // @todo
      'endpoint_name' => check_plain(arg(4)),
    ),
    array(
      'title' => $form_state['values']['odata_endpoint_title'],
      'url' => $form_state['values']['odata_endpoint_uri'],
      'options' => serialize($options),
    )
  );

  drupal_set_message(t('Your settings have been saved.'));
}
