<?php
/**
 * @file
 * The main file that describes the "overrides" of the original views.
 */

/**
 * Implements hook_views_data().
 */
function odata_views_data() {

  $data = array();

  $query = db_select('odata_endpoints', 'oe')
    ->fields('oe', array('title', 'url', 'options', 'endpoint_name'))
    ->execute();

  foreach ($query as $endpoint) {
    $data[$endpoint->endpoint_name]['table']['group'] = t('Open Data');
    $endpoint_options = unserialize($endpoint->options);

    $data[$endpoint->endpoint_name]['table']['base'] = array(
      // This is the identifier field for the view.
      'field' => $endpoint->url . '/' . $endpoint_options['collection'],
      'title' => $endpoint->title,
      'help' => t('Directly query oData:@title to get some data realtime.', array('@title' => $endpoint->title)),
      'query class' => 'odata_query',
      'accept_request_header' => $endpoint_options['accept_request_header'],
    );

    $data[$endpoint->endpoint_name]['table']['endpoint'] = $endpoint->url . '/' . $endpoint_options['collection'];
  }

  return $data;
}

/**
 * Implements hook_views_data_alter().
 */
function odata_views_data_alter(&$data) {

  $query = db_select('odata_endpoints', 'oe')
    ->fields('oe', array('title', 'url', 'options', 'endpoint_name'))
    ->execute();

  foreach ($query as $endpoint) {

    $endpoint_options = unserialize($endpoint->options);
    $schema = unserialize($endpoint_options['collections_schema']);
    $collection = $endpoint_options['collection'];

    $schema = _odata_return_fields($collection, $schema[$collection]);
    $data[$endpoint->endpoint_name] = array_merge($data[$endpoint->endpoint_name], $schema);

    // Override the combine filter.
    $data['views']['combine']['filter'] = array(
      'handler' => 'ODataHandlerFilterCombine',
    );
  }
}

/**
 * Implements hook_views_plugins().
 */
function odata_views_plugins() {
  return array(
    'query' => array(
      'odata_query' => array(
        'title' => t('oData query'),
        'help' => t('Query will be generated and executed using current oData endpoint.'),
        'handler' => 'OdataPluginQueryOdata',
      ),
    ),
  );
}

/**
 * Returns schema array to be inserted in the hook_views_data_alter() array.
 *
 * @param string $group
 *   Group name to group fields together.
 *
 * @param array $field
 *   Array with field properties.
 *
 * @return array
 *   Returns an array to be inserted in hook_views_data_alter().
 */
function _odata_return_fields($group, $field) {

  foreach ($field as $name => $attributes) {

    // Find the associated type.
    $type = odata_convert_type($attributes['Type']);

    $schema[$name] = array(
      'title' => $name,
      'group' => $group,
      'field' => array(
        'handler' => 'ODataHandlerField' . $type,
        'click sortable' => TRUE,
        'help' => $name,
      ),
      'sort' => array(
        'help' => t('Order results as @name.', array('@name' => $attributes['Name'])),
        'handler' => 'ODataHandlerSort' . $type,
      ),
      'filter' => array(
        'help' => t('Filter results by @name.', array('@name' => $attributes['Name'])),
        'handler' => 'ODataHandlerFilter' . $type,
      ),
      'argument' => array(
        'help' => t('Filter results by @name.', array('@name' => $attributes['Name'])),
        'handler' => 'ODataHandlerArgument' . $type,
      ),
    );

    // If it is a complex type, we need a recursion to handle its fields.
    if ($attributes['Type'] == 'ComplexType') {
      $schema[$name]['field']['complex_fields'] = $attributes['Complex'];
    }
  }

  return $schema;
}
